require ("zip");
require ("lxp");

local prog
local chan
local tag
local data
local grab
local lang
local UseCache

local function XMLDateToDate(str)
	local date={}
	date.Year=tonumber(string.sub(str,1,4))
	date.Month=tonumber(string.sub(str,5,6))
	date.Day=tonumber(string.sub(str,7,8))
	date.Hour=tonumber(string.sub(str,9,10))
	date.Minute=tonumber(string.sub(str,11,12))
	date.Second=tonumber(string.sub(str,13,14))
	if time.IsEUDST(date) then
		date.Hour=date.Hour-1
		date=time.Fix(date)
		end
	return date
	end

local function StartElement (parser, name, attrs)
	if name=="programme" then 
		prog={
				Channel=chan.ID,
				Start=XMLDateToDate(attrs["start"]),
				Stop=XMLDateToDate(attrs["stop"]),
				}
		end
	lang=attrs["lang"];
	data=""
	end

local function EndElement (parser, name)
	data=XGUI.ValidXML(data)
	if name=="programme" then 
		XGUI.AddProgram(grab,prog)
		prog=nil
		elseif name=="title" then prog.Title={Value=data,Lang=lang}
		elseif name=="desc" then prog.Description={Value=data,Lang=lang}
		elseif name=="category" then prog.Category={Value=data,Lang=lang}
		elseif name=="sub-title" then prog.Sub_Title={Value=data,Lang=lang}
		elseif prog then print(string.format("Unused element %s",name))
		end
	end

local function CharacterData (parser, str)
	data=data..str
	end

local function Pass1End(grabber)
	print("Clearing cache")
	XGUI.CleanCache(grabber,1);
	return XGUI.OK
	end

local ListingsURL="http://www.bleb.org/tv/data/listings?days=0..%d&format=XMLTV&channels=%s&file=zip"

local function Pass1Func(grabber,channel,maxdays)
	
	local callbacks = {
		StartElement = StartElement,
		EndElement = EndElement,
		CharacterData = CharacterData
		}
	local cacheid=nil
	local cahcemaxdays=nil
	if UseCache then
	   cacheid=channel.ID
	   cachemaxdays=1
	   end
		
	if maxdays==0 then maxdays=7
		end
	XGUI.SetStatus(2,"Downloading "..channel.Name)
	local page,err = XGUI.Get(string.format(ListingsURL,maxdays-1,channel.ID),grabber,_,_,cacheid,cachemaxdays)
	if err>0 then
		--sleep for 2 secs if file didnt come from cache
		XGUI.Sleep(2000)
		end
	if (err<400)and(err~=-1) then
		chan=channel
		grab=grabber
		local tmp=string.format("%s%s.zip",TEMP,channel.ID)
		local file=io.open(tmp,"wb")
		file:write(page);
		file:close();
		local zf=zip.open(tmp)
		file=zf:open("data.xml")
		local lx = lxp.new(callbacks)
		for l in file:lines() do
			lx:parse(l)
			lx:parse("\n")
			end
		lx:parse()
		lx:close()
		file:close()
		zf:close()
		return XGUI.OK
		else
			return XGUI.Fail
		end
	end

local function Pass1Start(grabber)
	UseCache=toboolean(XGUI.GetSetting(grabber,"UseCache",true))
	return XGUI.OK
	end

local function Init(grabber)
	XGUI.AddSetting(grabber,"UseCache",XGUI.SettingType.Boolean,"true","Use Cache")
	end

Grabbers.UK_Bleb={
	Name = "UK - Bleb",
	Version = "1.0",
	Encoding = XGUI.Encoding.UTF8,
	Author = "Alan Birtles",
	InfoURL = "http://www.birtles.org.uk/xmltv",
	SourceURL = "http://www.bleb.org/tv",
	Passes={
		Pass1={
			Type=XGUI.PassTypes.Channel,
			Start=Pass1Start,
			End=Pass1End,
			Func=Pass1Func,
			},
		},
	Init=Init,		
	}
